import re
import string
import time

def get_value(searchPhrase, entry):
    # get everything after searchPhrase
    try:
        value = re.split(searchPhrase, entry)[1]
    except IndexError:
        # cannot find value for searchPhrase
        return "None"
        
    # remove trailing key:value pairs
    value = re.split("\s+", value)[0]

    if not value:
        value = "None"

    return value


# This is not perfect, it will return everything up to the next
# known key. If a new key:value pair is added in the log file after
# one of the values that contain spaces then the new key:value pair
# will be returned as part of the spaced value.
def get_spaced_value(searchPhrase, entry, known_keys):
    # get everything after searchPhrase
    try:
        value = re.split(searchPhrase, entry)[1]
    except IndexError:
        # cannot find value for searchPhrase
        return "None"
    
    # remove trailing key:value pairs
    for key in known_keys:
        value = re.split(key, value)[0]
        
    value = value.strip()

    if not value:
        value = "None"

    return value


def none_to_zero(value):
    if value == "None":
        value = 0

    return value
                     

def parse_entry(entry):
    #
    # Be robust against offset pointing into the middle of a record,
    # which can happen when the service is stopped and not restarted
    # until the gridftp logfile has been rotated; in that case we
    # should process all entries from the start of the current file.
    #
    # If the very first entry is corrupt due to other causes, we will
    # keep hitting it, so we sleep a bit to avoid spinning the CPU.
    #
    
    if entry[0:6] != 'JOBID=':
        return "ERROR"
    
    known_keys = ["BKSERVER=", \
                  "CONDORID=", \
                  "DESTINATION=", \
                  "DONECODE=", \
                  "EXITCODE=", \
                  "JOBID=", \
                  "LASTUPDATETIME=", \
                  "NETWORKSERVER=", \
                  "OWNER=", \
                  "STATEENTERTIME=", \
                  "STATENAME=", \
                  "STATUSREASON=", \
                  "UIHOST=", \
                  "VO=" ]
    
    # simple values, no spaces
    BKserver = get_value("BKSERVER=", entry)
    Condor_Id = get_value("CONDORID=", entry)
    destination = get_value("DESTINATION=", entry)
    jobId = get_value("JOBID=", entry)
    lastUpdateTime = get_value("LASTUPDATETIME=", entry)
    NetWorkServer = get_value("NETWORKSERVER=", entry)
    stateEnterTime = get_value("STATEENTERTIME=", entry)
    state = get_value("STATENAME=", entry)
    uiHost = get_value("UIHOST=", entry)
    VO = get_value("VO=", entry)

    # arrr, values may contain spaces or equals
    # use more complex, slower parser
    StatusReason = get_spaced_value("STATUSREASON=", entry, known_keys)
    OWNER = get_spaced_value("OWNER=", entry, known_keys)

    # if the field was not found then the value will be "None"
    # this has to be converted to 0 for the numeric fields
    lastUpdateTime = none_to_zero(lastUpdateTime)
    stateEnterTime = none_to_zero(stateEnterTime)
    Condor_Id = none_to_zero(Condor_Id)

    # clean up StatusReason
    StatusReason=StatusReason.replace("\n","")
    StatusReason=StatusReason.replace(",","")
    StatusReason=StatusReason.replace("(","")
    StatusReason=StatusReason.replace(")","")

    rgma_time = map(lambda x: "%.2d"%x, time.gmtime(int(stateEnterTime)))
    Date = rgma_time[0] + "-" + rgma_time[1] + "-" + rgma_time[2]
    Time = rgma_time[3] + ":" + rgma_time[4] + ":" + rgma_time[5]
    
    thislist = []
    thislist.append("'"+jobId+"'")
    thislist.append("'"+VO+"'")
    thislist.append("'"+state+"'")
    thislist.append("'"+StatusReason+"'")
    thislist.append("'"+OWNER+"'")
    thislist.append("'"+BKserver+"'")
    thislist.append("'"+NetWorkServer+"'")
    thislist.append("'"+uiHost+"'")
    thislist.append("'"+destination+"'")
    thislist.append(Condor_Id)
    thislist.append("'" + stateEnterTime + "'" )
    thislist.append("'" + Date + "'")
    thislist.append("'" + Time + "'")

    command = "INSERT INTO JobStatusRaw(" + \
              "Job_id," + \
              "VO," + \
              "State," + \
              "StateInfo," + \
              "Owner," + \
              "BKServer," + \
              "NetworkServer," + \
              "UIHost," + \
              "Destination," + \
              "Condor_ID," + \
              "StateEnterTime," + \
              "MeasurementDate," + \
              "MeasurementTime ) VALUES ("
        
    for i in thislist:
        command = command + str(i) + ","
    command = command[0:-1] + ");"

    return command















